"""
Cloud Seeder CLI - Command line interface for cloud-init generation
MIT License
"""

import yaml
import argparse
import sys
import shutil
import subprocess
import logging
from pathlib import Path
from typing import Dict, Any, Optional, List
from dataclasses import dataclass

from .seeder import CloudInitConfig, User, WriteFile, SELinuxMode, generate_password_hash


@dataclass
class NodeConfig:
    """Configuration for a single node"""
    name: str
    hostname: str
    fqdn: str
    ip_address: str
    gateway: str
    netmask: str
    dns_servers: List[str]
    interface: str = "ens192"
    
    
class CloudSeederCLI:
    """Main CLI application for cloud-init seed generation"""
    
    def __init__(self, config_path: Path, output_dir: Path = Path("output"), 
                 log_level: str = "INFO"):
        self.config_path = config_path
        self.output_dir = output_dir
        self.config = None
        self._setup_logging(log_level)
        
    def _setup_logging(self, level: str):
        """Configure logging"""
        logging.basicConfig(
            level=getattr(logging, level.upper()),
            format='%(asctime)s - %(name)s - %(levelname)s - %(message)s'
        )
        self.logger = logging.getLogger('cloud-seeder')
        
    def load_config(self) -> Dict[str, Any]:
        """Load and validate configuration file"""
        self.logger.info(f"Loading configuration from {self.config_path}")
        
        try:
            with open(self.config_path) as f:
                self.config = yaml.safe_load(f)
        except Exception as e:
            self.logger.error(f"Failed to load config: {e}")
            raise
            
        # Validate structure
        self._validate_config()
        return self.config
        
    def _validate_config(self):
        """Validate configuration structure"""
        required_top = ["nodes"]
        for field in required_top:
            if field not in self.config:
                raise ValueError(f"Missing required top-level field: {field}")
                
        # Validate each node
        for node_name, node_config in self.config.get("nodes", {}).items():
            required_node = ["hostname", "network"]
            for field in required_node:
                if field not in node_config:
                    raise ValueError(f"Node {node_name} missing required field: {field}")
                    
    def _setup_directories(self):
        """Create output directory structure"""
        self.seeds_dir = self.output_dir / "seeds"
        self.images_dir = self.output_dir / "images"
        self.configs_dir = self.output_dir / "configs"
        
        for directory in [self.seeds_dir, self.images_dir, self.configs_dir]:
            directory.mkdir(parents=True, exist_ok=True)
            self.logger.debug(f"Created directory: {directory}")
            
    def _create_node_config(self, node_name: str, node_data: Dict[str, Any]) -> NodeConfig:
        """Create NodeConfig from raw configuration data"""
        network = node_data.get("network", {})
        
        # Handle DNS servers - can be string or list
        dns_servers = network.get("dns_servers", ["8.8.8.8"])
        if isinstance(dns_servers, str):
            dns_servers = [dns_servers]
            
        return NodeConfig(
            name=node_name,
            hostname=node_data.get("hostname", node_name),
            fqdn=node_data.get("fqdn", f"{node_data.get('hostname', node_name)}.local"),
            ip_address=network.get("ip_address", ""),
            gateway=network.get("gateway", ""),
            netmask=network.get("netmask", "255.255.255.0"),
            dns_servers=dns_servers,
            interface=network.get("interface", "ens192")
        )
        
    def generate_cloud_init(self, node_name: str, node_data: Dict[str, Any]) -> str:
        """Generate cloud-init configuration for a node"""
        self.logger.info(f"Generating cloud-init for node: {node_name}")
        
        node_config = self._create_node_config(node_name, node_data)
        config = CloudInitConfig()
        
        # Set hostname
        config.set_hostname(node_config.hostname, fqdn=node_config.fqdn)
        
        # Configure datasource
        datasource = self.config.get("datasource", {})
        if datasource:
            config.config.update(datasource)
            
        # Package management
        pkg_mgmt = self.config.get("package_management", {})
        config.set_package_management(
            update=pkg_mgmt.get("update", True),
            upgrade=pkg_mgmt.get("upgrade", False),
            reboot_if_required=pkg_mgmt.get("reboot_if_required", False)
        )
        
        # Add packages
        packages = self.config.get("packages", [])
        packages.extend(node_data.get("packages", []))
        if packages:
            config.add_packages(packages)
            
        # Configure users
        self._configure_users(config, node_data)
        
        # Configure SSH
        ssh_config = self.config.get("ssh", {})
        config.set_ssh_config(
            pwauth=ssh_config.get("password_auth", False),
            disable_root=ssh_config.get("disable_root", True)
        )
        
        # Write files
        self._add_write_files(config, node_data)
        
        # Boot commands
        for cmd in self.config.get("bootcmd", []):
            config.add_bootcmd(cmd)
        for cmd in node_data.get("bootcmd", []):
            config.add_bootcmd(cmd)
            
        # Run commands
        for cmd in self.config.get("runcmd", []):
            config.add_runcmd(cmd)
        for cmd in node_data.get("runcmd", []):
            config.add_runcmd(cmd)
            
        # Network configuration if static
        if node_config.ip_address:
            self._configure_network(config, node_config)
            
        # System configuration
        if "timezone" in self.config:
            config.set_timezone(self.config["timezone"])
        if "locale" in self.config:
            config.set_locale(self.config["locale"])
            
        # SELinux configuration
        if "selinux" in self.config:
            selinux = self.config["selinux"]
            mode = SELinuxMode(selinux.get("mode", "enforcing"))
            config.set_selinux(mode, selinux.get("type", "targeted"))
            
        # Final message
        final_msg = node_data.get("final_message", 
                                  self.config.get("final_message", 
                                                 f"Cloud-init completed for {node_config.hostname}"))
        config.set_final_message(final_msg)
        
        return config.generate()
        
    def _configure_users(self, config: CloudInitConfig, node_data: Dict[str, Any]):
        """Configure users for the node"""
        # Global users
        for user_data in self.config.get("users", []):
            user = self._create_user(user_data)
            config.add_user(user)
            
        # Node-specific users
        for user_data in node_data.get("users", []):
            user = self._create_user(user_data)
            config.add_user(user)
            
        # Default user customization
        default_user = self.config.get("default_user", {})
        if default_user:
            config.system_info["default_user"].update(default_user)
            
    def _create_user(self, user_data: Dict[str, Any]) -> User:
        """Create User object from configuration"""
        # Handle password - can be plaintext or hashed
        passwd = user_data.get("passwd")
        if passwd and not passwd.startswith("$"):
            # It's plaintext, hash it
            passwd = generate_password_hash(passwd)
            
        return User(
            name=user_data["name"],
            gecos=user_data.get("gecos", ""),
            primary_group=user_data.get("primary_group"),
            groups=user_data.get("groups", []),
            lock_passwd=user_data.get("lock_passwd", True),
            passwd=passwd,
            sudo=user_data.get("sudo", "ALL=(ALL) NOPASSWD:ALL"),
            shell=user_data.get("shell", "/bin/bash"),
            ssh_authorized_keys=user_data.get("ssh_authorized_keys", [])
        )
        
    def _add_write_files(self, config: CloudInitConfig, node_data: Dict[str, Any]):
        """Add write_files entries"""
        # Global files
        for file_data in self.config.get("write_files", []):
            config.add_write_file(WriteFile(**file_data))
            
        # Node-specific files
        for file_data in node_data.get("write_files", []):
            config.add_write_file(WriteFile(**file_data))
            
    def _configure_network(self, config: CloudInitConfig, node_config: NodeConfig):
        """Configure static networking"""
        # Add network configuration
        config.add_network_interface(
            interface=node_config.interface,
            dhcp4=False,
            addresses=[f"{node_config.ip_address}/24"],
            gateway4=node_config.gateway,
            nameservers={"addresses": node_config.dns_servers}
        )
        
    def create_seed_iso(self, node_name: str, user_data: str, 
                       node_config: Optional[NodeConfig] = None) -> bool:
        """Create cloud-init seed ISO"""
        iso_path = self.images_dir / f"{node_name}-seed.iso"
        seed_dir = self.seeds_dir / node_name
        seed_dir.mkdir(exist_ok=True)
        
        # Write user-data
        (seed_dir / "user-data").write_text(user_data)
        
        # Write meta-data
        meta_data = f"instance-id: {node_name}\nlocal-hostname: {node_name}\n"
        (seed_dir / "meta-data").write_text(meta_data)
        
        # Write network-config if we have static config
        if node_config and node_config.ip_address:
            network_config = {
                "version": 2,
                "ethernets": {
                    node_config.interface: {
                        "dhcp4": False,
                        "addresses": [f"{node_config.ip_address}/24"],
                        "gateway4": node_config.gateway,
                        "nameservers": {
                            "addresses": node_config.dns_servers
                        }
                    }
                }
            }
            (seed_dir / "network-config").write_text(yaml.dump(network_config))
            
        # Find ISO creation tool
        iso_tool = None
        for tool in ["genisoimage", "mkisofs", "xorrisofs"]:
            if shutil.which(tool):
                iso_tool = tool
                break
                
        if not iso_tool:
            self.logger.error("No ISO creation tool found. Install genisoimage or mkisofs")
            return False
            
        # Build ISO command
        if iso_tool == "genisoimage":
            cmd = [
                "genisoimage", "-output", str(iso_path),
                "-volid", "cidata", "-joliet", "-rock",
                str(seed_dir)
            ]
        elif iso_tool == "mkisofs":
            cmd = [
                "mkisofs", "-o", str(iso_path),
                "-V", "cidata", "-J", "-r",
                str(seed_dir)
            ]
        else:  # xorrisofs
            cmd = [
                "xorrisofs", "-o", str(iso_path),
                "-V", "cidata", "-J", "-r",
                str(seed_dir)
            ]
            
        # Create ISO
        try:
            subprocess.run(cmd, check=True, capture_output=True)
            self.logger.info(f"Created ISO: {iso_path}")
            return True
        except subprocess.CalledProcessError as e:
            self.logger.error(f"Failed to create ISO: {e}")
            return False
            
    def generate_all(self, create_isos: bool = False):
        """Generate configurations for all nodes"""
        self._setup_directories()
        
        for node_name, node_data in self.config.get("nodes", {}).items():
            # Generate cloud-init
            user_data = self.generate_cloud_init(node_name, node_data)
            
            # Save configuration
            config_path = self.configs_dir / f"{node_name}-cloud-init.yaml"
            config_path.write_text(user_data)
            self.logger.info(f"Saved configuration: {config_path}")
            
            # Create ISO if requested
            if create_isos:
                node_config = self._create_node_config(node_name, node_data)
                self.create_seed_iso(node_name, user_data, node_config)
                
    def clean(self):
        """Clean output directory"""
        if self.output_dir.exists():
            shutil.rmtree(self.output_dir)
            self.logger.info(f"Cleaned output directory: {self.output_dir}")


def main(args=None):
    """Main entry point for the CLI"""
    parser = argparse.ArgumentParser(
        prog="cloud-seeder",
        description="Cloud Seeder - Generate cloud-init configurations from YAML",
        epilog="Example: cloud-seeder config.yaml --iso"
    )
    parser.add_argument(
        "config",
        type=Path,
        help="Configuration file (YAML)"
    )
    parser.add_argument(
        "--output-dir",
        "-o",
        type=Path,
        default=Path("output"),
        help="Output directory (default: output)"
    )
    parser.add_argument(
        "--iso",
        "-i",
        action="store_true",
        help="Create seed ISOs"
    )
    parser.add_argument(
        "--clean",
        "-c",
        action="store_true",
        help="Clean output directory before generating"
    )
    parser.add_argument(
        "--log-level",
        "-l",
        choices=["DEBUG", "INFO", "WARNING", "ERROR"],
        default="INFO",
        help="Logging level"
    )
    parser.add_argument(
        "--validate-only",
        "-v",
        action="store_true",
        help="Only validate configuration without generating files"
    )
    parser.add_argument(
        "--version",
        action="version",
        version="%(prog)s 0.1.0"
    )
    
    # Parse arguments - use provided args for testing, otherwise sys.argv
    parsed_args = parser.parse_args(args)
    
    # Create CLI instance
    cli = CloudSeederCLI(parsed_args.config, parsed_args.output_dir, parsed_args.log_level)
    
    try:
        # Load configuration
        cli.load_config()
        
        if parsed_args.validate_only:
            print("Configuration is valid")
            return 0
            
        # Clean if requested
        if parsed_args.clean:
            cli.clean()
            
        # Generate configurations
        cli.generate_all(create_isos=parsed_args.iso)
        
        print(f"\nGeneration complete. Files in: {parsed_args.output_dir}")
        print(f"  - Configurations: {parsed_args.output_dir}/configs/")
        print(f"  - Seed files: {parsed_args.output_dir}/seeds/")
        if parsed_args.iso:
            print(f"  - ISO images: {parsed_args.output_dir}/images/")
            
    except Exception as e:
        print(f"Error: {e}", file=sys.stderr)
        return 1
        
    return 0


if __name__ == "__main__":
    sys.exit(main())