#!/usr/bin/env python3
"""
Cloud-Init Configuration Generator for Fedora/CentOS
MIT License
"""

import yaml
import hashlib
import random
import string
import subprocess
from typing import Dict, List, Any, Optional
from dataclasses import dataclass, field, asdict
from enum import Enum


class PowerMode(Enum):
    NONE = "none"
    POWEROFF = "poweroff"
    REBOOT = "reboot"
    HALT = "halt"


class SELinuxMode(Enum):
    ENFORCING = "enforcing"
    PERMISSIVE = "permissive"
    DISABLED = "disabled"


class TableType(Enum):
    MBR = "mbr"
    GPT = "gpt"


@dataclass
class User:
    name: str
    gecos: str = ""
    primary_group: Optional[str] = None
    groups: List[str] = field(default_factory=list)
    lock_passwd: bool = True
    passwd: Optional[str] = None
    sudo: Optional[str] = "ALL=(ALL) NOPASSWD:ALL"
    shell: str = "/bin/bash"
    ssh_authorized_keys: List[str] = field(default_factory=list)

    def to_dict(self) -> Dict[str, Any]:
        d = {"name": self.name}
        if self.gecos:
            d["gecos"] = self.gecos
        if self.primary_group:
            d["primary_group"] = self.primary_group
        if self.groups:
            d["groups"] = self.groups
        d["lock_passwd"] = self.lock_passwd
        if self.passwd:
            d["passwd"] = self.passwd
        if self.sudo is not None:
            d["sudo"] = self.sudo
        d["shell"] = self.shell
        if self.ssh_authorized_keys:
            d["ssh_authorized_keys"] = self.ssh_authorized_keys
        return d


@dataclass
class DiskSetup:
    device: str
    table_type: TableType = TableType.GPT
    layout: Any = True
    overwrite: bool = False

    def to_dict(self) -> Dict[str, Any]:
        return {
            "table_type": self.table_type.value,
            "layout": self.layout,
            "overwrite": self.overwrite,
        }


@dataclass
class FileSystem:
    label: str
    filesystem: str = "ext4"
    device: str = ""
    partition: str = "auto"
    overwrite: bool = False

    def to_dict(self) -> Dict[str, Any]:
        return asdict(self)


@dataclass
class Mount:
    device: str
    mount_point: str
    fs_type: str = "auto"
    options: str = "defaults,nofail"
    dump: str = "0"
    pass_num: str = "2"

    def to_list(self) -> List[str]:
        return [self.device, self.mount_point, self.fs_type, self.options, self.dump, self.pass_num]


@dataclass
class WriteFile:
    path: str
    content: str
    owner: str = "root:root"
    permissions: str = "0644"
    append: bool = False

    def to_dict(self) -> Dict[str, Any]:
        d = {
            "path": self.path,
            "owner": self.owner,
            "permissions": self.permissions,
            "content": self.content,
        }
        if self.append:
            d["append"] = True
        return d


@dataclass
class YumRepo:
    name: str
    baseurl: str
    enabled: bool = True
    gpgcheck: bool = True
    gpgkey: Optional[str] = None
    description: Optional[str] = None

    def to_dict(self) -> Dict[str, Any]:
        d = {"baseurl": self.baseurl, "enabled": self.enabled, "gpgcheck": self.gpgcheck}
        if self.gpgkey:
            d["gpgkey"] = self.gpgkey
        if self.description:
            d["name"] = self.description
        return d


class CloudInitConfig:
    def __init__(self):
        self.config = {"#cloud-config": None}
        self.system_info = {
            "distro": "fedora",
            "default_user": {
                "name": "fedora",
                "lock_passwd": True,
                "gecos": "Fedora Cloud User",
                "groups": ["wheel", "adm", "systemd-journal"],
                "sudo": ["ALL=(ALL) NOPASSWD:ALL"],
                "shell": "/bin/bash",
            },
        }
        self.packages = []
        self.users = []
        self.ssh_authorized_keys = []
        self.write_files = []
        self.runcmd = []
        self.bootcmd = []
        self.disk_setup = {}
        self.fs_setup = []
        self.mounts = []
        self.yum_repos = {}
        self.metadata = None
        self.network_config = None

    def set_hostname(self, hostname: str, fqdn: Optional[str] = None):
        self.config["hostname"] = hostname
        if fqdn:
            self.config["fqdn"] = fqdn

    def set_metadata(
        self,
        local_hostname: str,
        local_ipv4: str,
        ipv4_gateway: str,
        ipv4_netmask: str,
        dns_servers: str | List[str],
    ):
        """Set metadata for network configuration"""
        if isinstance(dns_servers, str):
            dns_servers = [dns_servers]

        self.metadata = {
            "local-hostname": local_hostname,
            "local-ipv4": local_ipv4,
            "ipv4-gateway": ipv4_gateway,
            "ipv4-netmask": ipv4_netmask,
            "dns-servers": dns_servers,
        }

    def set_network_config(
        self, version: int = 2, ethernets: Dict = None, renderer: str = "networkd"
    ):
        """Set network configuration v2"""
        if ethernets is None:
            ethernets = {}

        self.network_config = {"version": version, "renderer": renderer, "ethernets": ethernets}

    def add_network_interface(
        self,
        interface: str,
        dhcp4: bool = False,
        dhcp6: bool = False,
        addresses: List[str] = None,
        gateway4: str = None,
        nameservers: Dict = None,
    ):
        """Add network interface configuration"""
        if self.network_config is None:
            self.set_network_config()

        config = {"dhcp4": dhcp4, "dhcp6": dhcp6}

        if addresses:
            config["addresses"] = addresses
        if gateway4:
            config["gateway4"] = gateway4
        if nameservers:
            config["nameservers"] = nameservers

        self.network_config["ethernets"][interface] = config

    def set_package_management(
        self, update: bool = True, upgrade: bool = False, reboot_if_required: bool = False
    ):
        self.config["package_update"] = update
        self.config["package_upgrade"] = upgrade
        self.config["package_reboot_if_required"] = reboot_if_required

    def add_package(self, package: str):
        if package not in self.packages:
            self.packages.append(package)

    def add_packages(self, packages: List[str]):
        for pkg in packages:
            self.add_package(pkg)

    def add_user(self, user: User):
        self.users.append(user)

    def add_ssh_key(self, key: str):
        if key not in self.ssh_authorized_keys:
            self.ssh_authorized_keys.append(key)

    def set_ssh_config(self, pwauth: bool = False, disable_root: bool = True):
        self.config["ssh_pwauth"] = pwauth
        self.config["disable_root"] = disable_root

    def add_write_file(self, write_file: WriteFile):
        self.write_files.append(write_file)

    def add_runcmd(self, command: List[str] | str):
        self.runcmd.append(command)

    def add_bootcmd(self, command: List[str] | str):
        self.bootcmd.append(command)

    def add_disk_setup(self, device: str, disk: DiskSetup):
        self.disk_setup[device] = disk

    def add_filesystem(self, fs: FileSystem):
        self.fs_setup.append(fs)

    def add_mount(self, mount: Mount):
        self.mounts.append(mount)

    def add_yum_repo(self, repo_id: str, repo: YumRepo):
        self.yum_repos[repo_id] = repo

    def set_selinux(self, mode: SELinuxMode, selinux_type: str = "targeted"):
        selinux_config = WriteFile(
            path="/etc/selinux/config",
            content=f"SELINUX={mode.value}\nSELINUXTYPE={selinux_type}\n",
        )
        self.add_write_file(selinux_config)

    def add_firewall_service(self, service: str, permanent: bool = True):
        cmd = ["firewall-cmd"]
        if permanent:
            cmd.append("--permanent")
        cmd.extend(["--add-service", service])
        self.add_runcmd(cmd)
        if permanent:
            self.add_runcmd(["firewall-cmd", "--reload"])

    def add_firewall_port(self, port: int, protocol: str = "tcp", permanent: bool = True):
        cmd = ["firewall-cmd"]
        if permanent:
            cmd.append("--permanent")
        cmd.extend(["--add-port", f"{port}/{protocol}"])
        self.add_runcmd(cmd)
        if permanent:
            self.add_runcmd(["firewall-cmd", "--reload"])

    def add_systemd_service(
        self,
        name: str,
        description: str,
        exec_start: str,
        service_type: str = "simple",
        restart: str = "on-failure",
        user: str = "root",
        wanted_by: str = "multi-user.target",
        after: str = "network.target",
    ):
        service_content = f"""[Unit]
Description={description}
After={after}

[Service]
Type={service_type}
ExecStart={exec_start}
Restart={restart}
User={user}

[Install]
WantedBy={wanted_by}
"""
        service_file = WriteFile(
            path=f"/etc/systemd/system/{name}.service", content=service_content
        )
        self.add_write_file(service_file)
        self.add_runcmd(["systemctl", "daemon-reload"])
        self.add_runcmd(["systemctl", "enable", name])

    def set_timezone(self, timezone: str):
        self.config["timezone"] = timezone

    def set_locale(self, locale: str = "en_US.UTF-8"):
        self.config["locale"] = locale
        self.config["locale_configfile"] = "/etc/locale.conf"

    def set_final_message(self, message: str):
        self.config["final_message"] = message

    def set_power_state(self, mode: PowerMode, delay: str = "now", message: str = ""):
        if mode != PowerMode.NONE:
            self.config["power_state"] = {"mode": mode.value, "delay": delay, "message": message}

    def enable_debug_output(self):
        self.config["output"] = {"all": "| tee -a /var/log/cloud-init-output.log"}

    def generate(self) -> str:
        # Build the final configuration
        config = dict(self.config)

        # Add system info
        config["system_info"] = self.system_info

        # Add metadata if set
        if self.metadata:
            config["metadata"] = self.metadata

        # Add network config if set
        if self.network_config:
            config["network"] = self.network_config

        # Add packages
        if self.packages:
            config["packages"] = self.packages

        # Add users
        if self.users:
            config["users"] = [u.to_dict() for u in self.users]

        # Add SSH keys
        if self.ssh_authorized_keys:
            config["ssh_authorized_keys"] = self.ssh_authorized_keys

        # Add write files
        if self.write_files:
            config["write_files"] = [wf.to_dict() for wf in self.write_files]

        # Add commands
        if self.bootcmd:
            config["bootcmd"] = self.bootcmd
        if self.runcmd:
            config["runcmd"] = self.runcmd

        # Add storage config
        if self.disk_setup:
            config["disk_setup"] = {k: v.to_dict() for k, v in self.disk_setup.items()}
        if self.fs_setup:
            config["fs_setup"] = [fs.to_dict() for fs in self.fs_setup]
        if self.mounts:
            config["mounts"] = [m.to_list() for m in self.mounts]

        # Add yum repos
        if self.yum_repos:
            config["yum_repos"] = {k: v.to_dict() for k, v in self.yum_repos.items()}

        # Remove the placeholder
        del config["#cloud-config"]

        # Custom representer for literal strings
        def literal_str_representer(dumper, data):
            if "\n" in data:
                return dumper.represent_scalar("tag:yaml.org,2002:str", data, style="|")
            return dumper.represent_scalar("tag:yaml.org,2002:str", data)

        # Add the representer for strings
        yaml.add_representer(str, literal_str_representer)

        # Generate YAML with proper formatting
        yaml_content = yaml.dump(
            config, default_flow_style=False, sort_keys=False, allow_unicode=True, width=120
        )
        return "#cloud-config\n" + yaml_content


def generate_password_hash(password: str, method: str = "SHA512") -> str:
    """
    Generate password hash for cloud-init using system tools
    Falls back to different methods based on what's available
    """
    # Method 1: Try using openssl (most universal)
    if method == "SHA512":
        try:
            # Generate random salt
            salt = "".join(random.choices(string.ascii_letters + string.digits, k=16))
            # Use openssl to generate SHA512 hash
            cmd = ["openssl", "passwd", "-6", "-salt", salt, password]
            result = subprocess.run(cmd, capture_output=True, text=True, check=True)
            return result.stdout.strip()
        except (subprocess.CalledProcessError, FileNotFoundError):
            pass

    # Method 2: Try using mkpasswd if available
    try:
        cmd = ["mkpasswd", "-m", "sha-512", password]
        result = subprocess.run(cmd, capture_output=True, text=True, check=True)
        return result.stdout.strip()
    except (subprocess.CalledProcessError, FileNotFoundError):
        pass

    # Method 3: Try using python's passlib if installed
    try:
        from passlib.hash import sha512_crypt

        return sha512_crypt.hash(password)
    except ImportError:
        pass

    # Method 4: Try using htpasswd with bcrypt
    try:
        cmd = ["htpasswd", "-nbB", "user", password]
        result = subprocess.run(cmd, capture_output=True, text=True, check=True)
        # Extract just the hash part
        return result.stdout.strip().split(":", 1)[1]
    except (subprocess.CalledProcessError, FileNotFoundError):
        pass

    # Method 5: Use Python's hashlib for a basic hash (not recommended for production)
    print("WARNING: Using basic hash method. Install 'openssl' or 'mkpasswd' for secure hashes.")
    print("On Fedora: sudo dnf install openssl-devel mkpasswd")
    # This is NOT a proper Unix crypt hash, but better than plaintext
    salt = "".join(random.choices(string.ascii_letters + string.digits, k=16))
    hash_input = f"{salt}{password}".encode()
    hash_output = hashlib.sha512(hash_input).hexdigest()
    # Format it to look like a SHA512 crypt (but it's not compatible)
    return f"$6$WEAK${salt}${hash_output}"


# Example usage functions
def create_basic_web_server():
    """Example: Basic web server configuration"""
    config = CloudInitConfig()

    # System settings
    config.set_hostname("webserver")
    config.set_package_management(update=True, upgrade=True)
    config.enable_debug_output()

    # Packages
    config.add_packages(["nginx", "certbot", "python3-certbot-nginx"])

    # Users
    admin = User(
        name="admin",
        gecos="Web Administrator",
        groups=["wheel"],
        ssh_authorized_keys=["ssh-rsa AAAAB3... admin@example.com"],
    )
    config.add_user(admin)

    # Firewall
    config.add_firewall_service("http")
    config.add_firewall_service("https")

    # Nginx configuration
    nginx_config = WriteFile(
        path="/etc/nginx/conf.d/default.conf",
        content="""server {
    listen 80;
    server_name _;
    root /var/www/html;
    index index.html;

    location / {
        try_files $uri $uri/ =404;
    }
}""",
    )
    config.add_write_file(nginx_config)

    # Start services
    config.add_runcmd(["systemctl", "enable", "nginx"])
    config.add_runcmd(["systemctl", "start", "nginx"])

    # SELinux
    config.set_selinux(SELinuxMode.ENFORCING)

    return config.generate()


def create_development_environment():
    """Example: Development environment configuration"""
    config = CloudInitConfig()

    # System settings
    config.set_hostname("dev-machine")
    config.set_timezone("America/New_York")
    config.set_locale("en_US.UTF-8")

    # Development packages
    dev_packages = [
        "git",
        "vim-enhanced",
        "tmux",
        "htop",
        "python3",
        "python3-pip",
        "python3-virtualenv",
        "nodejs",
        "npm",
        "podman",
        "buildah",
        "@development-tools",
    ]
    config.add_packages(dev_packages)

    # Developer user
    developer = User(
        name="developer",
        gecos="Developer Account",
        groups=["wheel", "podman"],
        passwd=generate_password_hash("changeme123"),
        lock_passwd=False,
        ssh_authorized_keys=["ssh-rsa AAAAB3... dev@example.com"],
    )
    config.add_user(developer)

    # Enable SSH password auth for development
    config.set_ssh_config(pwauth=True, disable_root=False)

    # Git configuration
    git_config = WriteFile(
        path="/home/developer/.gitconfig",
        owner="developer:developer",
        content="""[user]
    name = Developer
    email = dev@example.com
[core]
    editor = vim
[alias]
    st = status
    co = checkout
    br = branch
    cm = commit
    ps = push
""",
    )
    config.add_write_file(git_config)

    # Bash aliases
    bash_aliases = WriteFile(
        path="/home/developer/.bash_aliases",
        owner="developer:developer",
        content="""# Development aliases
alias ll='ls -la'
alias gs='git status'
alias gc='git commit'
alias gp='git push'
alias gco='git checkout'
alias gbr='git branch'
alias py='python3'
alias venv='python3 -m venv'
alias act='source venv/bin/activate'
""",
    )
    config.add_write_file(bash_aliases)

    # Source aliases in bashrc
    bashrc_update = WriteFile(
        path="/home/developer/.bashrc",
        owner="developer:developer",
        append=True,
        content="\n# Load aliases\n[ -f ~/.bash_aliases ] && . ~/.bash_aliases\n",
    )
    config.add_write_file(bashrc_update)

    # Set up development directories
    config.add_runcmd(["mkdir", "-p", "/home/developer/projects"])
    config.add_runcmd(["chown", "developer:developer", "/home/developer/projects"])

    return config.generate()


def create_container_host():
    """Example: Container host with Podman"""
    config = CloudInitConfig()

    # System settings
    config.set_hostname("container-host")
    config.set_package_management(update=True, upgrade=True)

    # Container packages
    config.add_packages(["podman", "buildah", "skopeo", "podman-compose"])

    # Storage for containers
    config.add_disk_setup(
        "/dev/vdb",
        DiskSetup(device="/dev/vdb", table_type=TableType.GPT, layout=True, overwrite=True),
    )

    config.add_filesystem(
        FileSystem(label="containers", filesystem="xfs", device="/dev/vdb", partition="1")
    )

    config.add_mount(Mount(device="/dev/vdb1", mount_point="/var/lib/containers", fs_type="xfs"))

    # Container user
    container_user = User(
        name="poduser",
        gecos="Container User",
        groups=["podman"],
        ssh_authorized_keys=["ssh-rsa AAAAB3... container@example.com"],
    )
    config.add_user(container_user)

    # Enable lingering for rootless containers
    config.add_runcmd(["loginctl", "enable-linger", "poduser"])

    # Podman registries configuration
    registries_conf = WriteFile(
        path="/etc/containers/registries.conf",
        content="""unqualified-search-registries = ["docker.io", "quay.io"]

[[registry]]
prefix = "docker.io"
location = "docker.io"

[[registry]]
prefix = "quay.io"
location = "quay.io"
""",
    )
    config.add_write_file(registries_conf)

    # Increase user namespaces
    sysctl_conf = WriteFile(
        path="/etc/sysctl.d/99-containers.conf", content="user.max_user_namespaces = 15000\n"
    )
    config.add_write_file(sysctl_conf)
    config.add_runcmd(["sysctl", "--system"])

    return config.generate()


if __name__ == "__main__":
    # Example: Generate configurations
    print("=== Web Server Configuration ===")
    print(create_basic_web_server())
    print("\n=== Development Environment Configuration ===")
    print(create_development_environment())
    print("\n=== Container Host Configuration ===")
    print(create_container_host())
