"""
Tests for cloud_seeder.cli module
"""

import pytest
import yaml
import tempfile
from pathlib import Path
from cloud_seeder.cli import CloudSeederCLI


@pytest.fixture
def sample_config():
    """Sample configuration for testing"""
    return {
        "timezone": "UTC",
        "packages": ["vim", "git"],
        "nodes": {
            "test-node": {
                "hostname": "test",
                "network": {
                    "ip_address": "10.0.0.10",
                    "gateway": "10.0.0.1",
                    "dns_servers": ["8.8.8.8"],
                },
            }
        },
    }


@pytest.fixture
def config_file(sample_config):
    """Create a temporary config file"""
    with tempfile.NamedTemporaryFile(mode="w", suffix=".yaml", delete=False) as f:
        yaml.dump(sample_config, f)
        config_path = Path(f.name)

    yield config_path

    # Cleanup
    config_path.unlink()


def test_cli_init(config_file):
    """Test CLI initialization"""
    cli = CloudSeederCLI(config_file)
    assert cli.config_path == config_file
    assert cli.output_dir == Path("output")


def test_load_config(config_file, sample_config):
    """Test configuration loading"""
    cli = CloudSeederCLI(config_file)
    loaded_config = cli.load_config()

    assert loaded_config["timezone"] == sample_config["timezone"]
    assert "test-node" in loaded_config["nodes"]


def test_validate_config_missing_nodes():
    """Test validation with missing nodes"""
    with tempfile.NamedTemporaryFile(mode="w", suffix=".yaml") as f:
        yaml.dump({"timezone": "UTC"}, f)
        f.flush()

        cli = CloudSeederCLI(Path(f.name))
        with pytest.raises(ValueError, match="Missing required top-level field: nodes"):
            cli.load_config()


def test_validate_config_missing_node_fields():
    """Test validation with missing node fields"""
    config = {
        "nodes": {
            "bad-node": {
                "hostname": "bad"
                # Missing network
            }
        }
    }

    with tempfile.NamedTemporaryFile(mode="w", suffix=".yaml") as f:
        yaml.dump(config, f)
        f.flush()

        cli = CloudSeederCLI(Path(f.name))
        with pytest.raises(ValueError, match="missing required field: network"):
            cli.load_config()


def test_create_node_config(config_file):
    """Test NodeConfig creation"""
    cli = CloudSeederCLI(config_file)
    cli.load_config()

    node_data = cli.config["nodes"]["test-node"]
    node_config = cli._create_node_config("test-node", node_data)

    assert node_config.name == "test-node"
    assert node_config.hostname == "test"
    assert node_config.ip_address == "10.0.0.10"
    assert node_config.gateway == "10.0.0.1"
    assert node_config.dns_servers == ["8.8.8.8"]


def test_generate_cloud_init(config_file):
    """Test cloud-init generation"""
    cli = CloudSeederCLI(config_file)
    cli.load_config()

    node_data = cli.config["nodes"]["test-node"]
    cloud_init = cli.generate_cloud_init("test-node", node_data)

    assert "#cloud-config" in cloud_init
    assert "hostname: test" in cloud_init
    assert "vim" in cloud_init
    assert "git" in cloud_init


def test_generate_all(config_file, tmp_path):
    """Test generating all configurations"""
    cli = CloudSeederCLI(config_file, output_dir=tmp_path)
    cli.load_config()
    cli.generate_all()

    # Check directories were created
    assert (tmp_path / "configs").exists()
    assert (tmp_path / "seeds").exists()
    assert (tmp_path / "images").exists()

    # Check config file was created
    config_file = tmp_path / "configs" / "test-node-cloud-init.yaml"
    assert config_file.exists()

    # Verify content
    content = config_file.read_text()
    assert "#cloud-config" in content
    assert "hostname: test" in content


def test_clean(config_file, tmp_path):
    """Test cleaning output directory"""
    cli = CloudSeederCLI(config_file, output_dir=tmp_path / "test_output")
    cli.load_config()
    cli.generate_all()

    # Verify files exist
    assert (tmp_path / "test_output").exists()

    # Clean
    cli.clean()

    # Verify cleaned
    assert not (tmp_path / "test_output").exists()
