"""
Tests for password hash generation
"""

import subprocess
from unittest.mock import patch, MagicMock
from cloud_seeder import generate_password_hash


@patch('subprocess.run')
def test_password_hash_openssl_success(mock_run):
    """Test password hash with openssl success"""
    mock_run.return_value = MagicMock(
        stdout="$6$salt$hashedpassword\n",
        returncode=0
    )
    
    result = generate_password_hash("testpass")
    
    assert result == "$6$salt$hashedpassword"
    assert mock_run.called
    args = mock_run.call_args[0][0]
    assert args[0] == "openssl"
    assert args[1] == "passwd"
    assert args[2] == "-6"


@patch('subprocess.run')
def test_password_hash_mkpasswd_fallback(mock_run):
    """Test password hash falls back to mkpasswd"""
    # First call (openssl) fails
    # Second call (mkpasswd) succeeds
    mock_run.side_effect = [
        subprocess.CalledProcessError(1, ["openssl"]),
        MagicMock(stdout="$6$mkpasswd$hash\n", returncode=0)
    ]
    
    result = generate_password_hash("testpass")
    
    assert result == "$6$mkpasswd$hash"
    assert mock_run.call_count == 2
    # Check mkpasswd was called
    second_call_args = mock_run.call_args_list[1][0][0]
    assert second_call_args[0] == "mkpasswd"


@patch('subprocess.run')
def test_password_hash_passlib_fallback(mock_run):
    """Test password hash falls back to passlib"""
    # All subprocess calls fail
    mock_run.side_effect = subprocess.CalledProcessError(1, ["cmd"])
    
    # Mock passlib at the import level
    import sys
    mock_passlib = MagicMock()
    mock_passlib.hash.sha512_crypt.hash.return_value = "$6$passlib$hash"
    sys.modules['passlib'] = mock_passlib
    sys.modules['passlib.hash'] = mock_passlib.hash
    
    try:
        result = generate_password_hash("testpass")
        assert result == "$6$passlib$hash"
    finally:
        # Clean up
        if 'passlib' in sys.modules:
            del sys.modules['passlib']
        if 'passlib.hash' in sys.modules:
            del sys.modules['passlib.hash']


@patch('subprocess.run')
def test_password_hash_htpasswd_fallback(mock_run):
    """Test password hash falls back to htpasswd"""
    # First three methods fail, htpasswd succeeds
    mock_run.side_effect = [
        subprocess.CalledProcessError(1, ["openssl"]),
        subprocess.CalledProcessError(1, ["mkpasswd"]),
        MagicMock(stdout="user:$2y$05$bcrypthash\n", returncode=0)
    ]
    
    # Make sure passlib is not available
    import sys
    if 'passlib' in sys.modules:
        del sys.modules['passlib']
    if 'passlib.hash' in sys.modules:
        del sys.modules['passlib.hash']
    
    result = generate_password_hash("testpass")
    
    assert result == "$2y$05$bcrypthash"


@patch('subprocess.run')
def test_password_hash_final_fallback(mock_run):
    """Test password hash final fallback (basic hash)"""
    # All subprocess methods fail
    mock_run.side_effect = subprocess.CalledProcessError(1, ["cmd"])
    
    # Make sure passlib is not available
    import sys
    if 'passlib' in sys.modules:
        del sys.modules['passlib']
    if 'passlib.hash' in sys.modules:
        del sys.modules['passlib.hash']
    
    result = generate_password_hash("testpass")
    
    # Should use basic hash method
    assert result.startswith("$6$WEAK$")
    assert len(result) > 20


@patch('subprocess.run')
def test_password_hash_filenotfound(mock_run):
    """Test password hash with FileNotFoundError"""
    # Simulate command not found
    mock_run.side_effect = FileNotFoundError("Command not found")
    
    # Make sure passlib is not available
    import sys
    if 'passlib' in sys.modules:
        del sys.modules['passlib']
    if 'passlib.hash' in sys.modules:
        del sys.modules['passlib.hash']
    
    result = generate_password_hash("testpass")
    
    # Should use basic hash method
    assert result.startswith("$6$WEAK$")