"""
Tests for cloud_seeder.seeder module
"""


from cloud_seeder import (
    CloudInitConfig,
    User,
    WriteFile,
    SELinuxMode,
    generate_password_hash,
)


def test_cloud_init_config_basic():
    """Test basic CloudInitConfig functionality"""
    config = CloudInitConfig()
    config.set_hostname("test-host")

    yaml_output = config.generate()
    assert "#cloud-config" in yaml_output
    assert "hostname: test-host" in yaml_output


def test_user_creation():
    """Test User dataclass creation"""
    user = User(
        name="testuser", groups=["wheel", "admin"], ssh_authorized_keys=["ssh-rsa AAAAB3..."]
    )

    user_dict = user.to_dict()
    assert user_dict["name"] == "testuser"
    assert "wheel" in user_dict["groups"]
    assert len(user_dict["ssh_authorized_keys"]) == 1


def test_write_file_creation():
    """Test WriteFile dataclass"""
    file_obj = WriteFile(
        path="/etc/test.conf", content="test content", owner="root:root", permissions="0644"
    )

    file_dict = file_obj.to_dict()
    assert file_dict["path"] == "/etc/test.conf"
    assert file_dict["content"] == "test content"
    assert file_dict["permissions"] == "0644"


def test_package_management():
    """Test package management configuration"""
    config = CloudInitConfig()
    config.set_package_management(update=True, upgrade=True, reboot_if_required=False)
    config.add_packages(["nginx", "git", "vim"])

    yaml_output = config.generate()
    assert "package_update: true" in yaml_output
    assert "package_upgrade: true" in yaml_output
    assert "nginx" in yaml_output
    assert "git" in yaml_output


def test_network_configuration():
    """Test network interface configuration"""
    config = CloudInitConfig()
    config.add_network_interface(
        interface="eth0",
        dhcp4=False,
        addresses=["192.168.1.100/24"],
        gateway4="192.168.1.1",
        nameservers={"addresses": ["8.8.8.8", "8.8.4.4"]},
    )

    yaml_output = config.generate()
    assert "ethernets:" in yaml_output
    assert "eth0:" in yaml_output
    assert "192.168.1.100/24" in yaml_output


def test_selinux_configuration():
    """Test SELinux configuration"""
    config = CloudInitConfig()
    config.set_selinux(SELinuxMode.ENFORCING)

    yaml_output = config.generate()
    assert "SELINUX=enforcing" in yaml_output
    assert "SELINUXTYPE=targeted" in yaml_output


def test_password_hash_generation():
    """Test password hash generation"""
    # This will use whatever method is available
    hash_result = generate_password_hash("testpassword")

    # Should start with $ (indicating it's hashed)
    assert hash_result.startswith("$")
    # Should have multiple $ separators
    assert hash_result.count("$") >= 2


def test_full_configuration():
    """Test a complete configuration"""
    config = CloudInitConfig()

    # System settings
    config.set_hostname("web-server", fqdn="web-server.example.com")
    config.set_timezone("America/New_York")
    config.set_locale("en_US.UTF-8")

    # Packages
    config.add_packages(["nginx", "certbot"])

    # Users
    admin_user = User(name="admin", groups=["wheel"], ssh_authorized_keys=["ssh-rsa AAAAB3..."])
    config.add_user(admin_user)

    # Files
    nginx_conf = WriteFile(path="/etc/nginx/sites-enabled/default", content="server { listen 80; }")
    config.add_write_file(nginx_conf)

    # Commands
    config.add_runcmd("systemctl enable nginx")
    config.add_runcmd("systemctl start nginx")

    yaml_output = config.generate()

    # Verify all components are present
    assert "hostname: web-server" in yaml_output
    assert "fqdn: web-server.example.com" in yaml_output
    assert "timezone: America/New_York" in yaml_output
    assert "nginx" in yaml_output
    assert "name: admin" in yaml_output
    assert "/etc/nginx/sites-enabled/default" in yaml_output
    assert "systemctl enable nginx" in yaml_output
