"""
Extended tests for cloud_seeder.seeder module to improve coverage
"""

import pytest
from cloud_seeder import (
    CloudInitConfig,
    User,
    WriteFile,
    DiskSetup,
    FileSystem,
    Mount,
    YumRepo,
    PowerMode,
    SELinuxMode,
    TableType,
)


def test_disk_setup():
    """Test DiskSetup configuration"""
    disk = DiskSetup(
        device="/dev/vdb",
        table_type=TableType.GPT,
        layout=True,
        overwrite=True
    )
    
    disk_dict = disk.to_dict()
    assert disk_dict["table_type"] == "gpt"
    assert disk_dict["layout"] is True
    assert disk_dict["overwrite"] is True


def test_filesystem():
    """Test FileSystem configuration"""
    fs = FileSystem(
        label="data",
        filesystem="xfs",
        device="/dev/vdb",
        partition="1"
    )
    
    fs_dict = fs.to_dict()
    assert fs_dict["label"] == "data"
    assert fs_dict["filesystem"] == "xfs"
    assert fs_dict["device"] == "/dev/vdb"
    assert fs_dict["partition"] == "1"


def test_mount():
    """Test Mount configuration"""
    mount = Mount(
        device="/dev/vdb1",
        mount_point="/data",
        fs_type="xfs"
    )
    
    mount_list = mount.to_list()
    assert mount_list[0] == "/dev/vdb1"
    assert mount_list[1] == "/data"
    assert mount_list[2] == "xfs"


def test_yum_repo():
    """Test YumRepo configuration"""
    repo = YumRepo(
        name="custom-repo",
        baseurl="https://example.com/repo",
        enabled=True,
        gpgcheck=True,
        gpgkey="https://example.com/repo/key.gpg",
        description="Custom Repository"
    )
    
    repo_dict = repo.to_dict()
    assert repo_dict["baseurl"] == "https://example.com/repo"
    assert repo_dict["enabled"] is True
    assert repo_dict["gpgkey"] == "https://example.com/repo/key.gpg"
    assert repo_dict["name"] == "Custom Repository"


def test_cloud_init_metadata():
    """Test metadata configuration"""
    config = CloudInitConfig()
    config.set_metadata(
        local_hostname="test-host",
        local_ipv4="192.168.1.100",
        ipv4_gateway="192.168.1.1",
        ipv4_netmask="255.255.255.0",
        dns_servers="8.8.8.8"
    )
    
    yaml_output = config.generate()
    assert "metadata:" in yaml_output
    assert "local-hostname: test-host" in yaml_output
    assert "192.168.1.100" in yaml_output


def test_cloud_init_metadata_multiple_dns():
    """Test metadata with multiple DNS servers"""
    config = CloudInitConfig()
    config.set_metadata(
        local_hostname="test-host",
        local_ipv4="192.168.1.100",
        ipv4_gateway="192.168.1.1",
        ipv4_netmask="255.255.255.0",
        dns_servers=["8.8.8.8", "8.8.4.4"]
    )
    
    yaml_output = config.generate()
    assert "8.8.8.8" in yaml_output
    assert "8.8.4.4" in yaml_output


def test_bootcmd():
    """Test boot commands"""
    config = CloudInitConfig()
    config.add_bootcmd("echo 'Starting boot'")
    config.add_bootcmd(["mkdir", "-p", "/test"])
    
    yaml_output = config.generate()
    assert "bootcmd:" in yaml_output
    assert "echo 'Starting boot'" in yaml_output
    assert "mkdir" in yaml_output


def test_disk_and_filesystem_setup():
    """Test disk and filesystem configuration"""
    config = CloudInitConfig()
    
    # Add disk
    disk = DiskSetup(
        device="/dev/vdb",
        table_type=TableType.MBR,
        layout=True
    )
    config.add_disk_setup("/dev/vdb", disk)
    
    # Add filesystem
    fs = FileSystem(
        label="data",
        filesystem="ext4",
        device="/dev/vdb",
        partition="1"
    )
    config.add_filesystem(fs)
    
    # Add mount
    mount = Mount(
        device="/dev/vdb1",
        mount_point="/data"
    )
    config.add_mount(mount)
    
    yaml_output = config.generate()
    assert "disk_setup:" in yaml_output
    assert "fs_setup:" in yaml_output
    assert "mounts:" in yaml_output


def test_yum_repos():
    """Test yum repository configuration"""
    config = CloudInitConfig()
    
    repo = YumRepo(
        name="epel",
        baseurl="https://example.com/epel",
        enabled=True
    )
    config.add_yum_repo("epel", repo)
    
    yaml_output = config.generate()
    assert "yum_repos:" in yaml_output
    assert "epel:" in yaml_output
    assert "baseurl: https://example.com/epel" in yaml_output


def test_firewall_services():
    """Test firewall service configuration"""
    config = CloudInitConfig()
    config.add_firewall_service("http")
    config.add_firewall_service("https", permanent=False)
    
    yaml_output = config.generate()
    assert "firewall-cmd" in yaml_output
    assert "--add-service" in yaml_output
    assert "http" in yaml_output
    assert "https" in yaml_output


def test_firewall_ports():
    """Test firewall port configuration"""
    config = CloudInitConfig()
    config.add_firewall_port(8080)
    config.add_firewall_port(9090, protocol="udp")
    
    yaml_output = config.generate()
    assert "8080/tcp" in yaml_output
    assert "9090/udp" in yaml_output


def test_systemd_service():
    """Test systemd service creation"""
    config = CloudInitConfig()
    config.add_systemd_service(
        name="myapp",
        description="My Application",
        exec_start="/usr/bin/myapp",
        service_type="forking",
        restart="always",
        user="myapp"
    )
    
    yaml_output = config.generate()
    assert "/etc/systemd/system/myapp.service" in yaml_output
    assert "Description=My Application" in yaml_output
    assert "ExecStart=/usr/bin/myapp" in yaml_output
    assert "Type=forking" in yaml_output
    assert "Restart=always" in yaml_output
    assert "User=myapp" in yaml_output


def test_timezone_and_locale():
    """Test timezone and locale settings"""
    config = CloudInitConfig()
    config.set_timezone("Europe/London")
    config.set_locale("en_GB.UTF-8")
    
    yaml_output = config.generate()
    assert "timezone: Europe/London" in yaml_output
    assert "locale: en_GB.UTF-8" in yaml_output


def test_final_message():
    """Test final message"""
    config = CloudInitConfig()
    config.set_final_message("System setup complete!")
    
    yaml_output = config.generate()
    assert "final_message: System setup complete!" in yaml_output


def test_power_state():
    """Test power state configuration"""
    config = CloudInitConfig()
    config.set_power_state(
        mode=PowerMode.REBOOT,
        delay="+5",
        message="Rebooting in 5 minutes"
    )
    
    yaml_output = config.generate()
    assert "power_state:" in yaml_output
    assert "mode: reboot" in yaml_output
    assert "delay: '+5'" in yaml_output or "delay: +5" in yaml_output  # Handle both quoted and unquoted
    assert "message: Rebooting in 5 minutes" in yaml_output


def test_power_state_none():
    """Test power state with NONE mode"""
    config = CloudInitConfig()
    config.set_power_state(PowerMode.NONE)
    
    yaml_output = config.generate()
    assert "power_state:" not in yaml_output


def test_debug_output():
    """Test debug output configuration"""
    config = CloudInitConfig()
    config.enable_debug_output()
    
    yaml_output = config.generate()
    assert "output:" in yaml_output
    assert "/var/log/cloud-init-output.log" in yaml_output


def test_network_config_version():
    """Test network configuration with custom version"""
    config = CloudInitConfig()
    config.set_network_config(
        version=1,
        ethernets={"eth0": {"dhcp4": True}},
        renderer="NetworkManager"
    )
    
    yaml_output = config.generate()
    assert "version: 1" in yaml_output
    assert "renderer: NetworkManager" in yaml_output


def test_user_without_sudo():
    """Test user creation without sudo"""
    user = User(
        name="limited",
        groups=["users"],
        sudo=None
    )
    
    user_dict = user.to_dict()
    assert "sudo" not in user_dict


def test_user_with_custom_shell():
    """Test user with custom shell"""
    user = User(
        name="fish-user",
        shell="/usr/bin/fish"
    )
    
    user_dict = user.to_dict()
    assert user_dict["shell"] == "/usr/bin/fish"


def test_write_file_append():
    """Test write file with append mode"""
    file_obj = WriteFile(
        path="/etc/motd",
        content="\nWelcome!",
        append=True
    )
    
    file_dict = file_obj.to_dict()
    assert file_dict["append"] is True


def test_ssh_keys_global():
    """Test global SSH keys"""
    config = CloudInitConfig()
    config.add_ssh_key("ssh-rsa AAAAB3... key1")
    config.add_ssh_key("ssh-rsa AAAAB3... key2")
    # Try to add duplicate
    config.add_ssh_key("ssh-rsa AAAAB3... key1")
    
    yaml_output = config.generate()
    assert "ssh_authorized_keys:" in yaml_output
    # Should only have 2 keys (no duplicates)
    assert yaml_output.count("ssh-rsa") == 2


def test_distro_configuration():
    """Test distro-specific configuration"""
    config = CloudInitConfig()
    config.system_info["distro"] = "centos"
    
    yaml_output = config.generate()
    assert "distro: centos" in yaml_output