"""
Extended tests for cloud_seeder.cli module to improve coverage
"""

import yaml
import tempfile
import subprocess
from pathlib import Path
from unittest.mock import patch, MagicMock
from cloud_seeder.cli import CloudSeederCLI, NodeConfig, main


def test_cli_with_users():
    """Test CLI with user configuration"""
    config = {
        "users": [
            {
                "name": "testuser",
                "passwd": "plaintext123",  # Should be hashed
                "groups": ["wheel"]
            }
        ],
        "nodes": {
            "test": {
                "hostname": "test",
                "network": {"interface": "eth0"},
                "users": [
                    {
                        "name": "nodeuser",
                        "passwd": "$6$already$hashed",  # Already hashed
                    }
                ]
            }
        }
    }
    
    with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml') as f:
        yaml.dump(config, f)
        f.flush()
        
        cli = CloudSeederCLI(Path(f.name))
        cli.load_config()
        cloud_init = cli.generate_cloud_init("test", config["nodes"]["test"])
        
        assert "testuser" in cloud_init
        assert "nodeuser" in cloud_init
        # Plain password should be hashed
        assert "plaintext123" not in cloud_init
        assert "$6$already$hashed" in cloud_init


def test_cli_with_default_user():
    """Test CLI with default user customization"""
    config = {
        "default_user": {
            "name": "custom",
            "groups": ["docker", "podman"]
        },
        "nodes": {
            "test": {
                "hostname": "test",
                "network": {}
            }
        }
    }
    
    with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml') as f:
        yaml.dump(config, f)
        f.flush()
        
        cli = CloudSeederCLI(Path(f.name))
        cli.load_config()
        cloud_init = cli.generate_cloud_init("test", config["nodes"]["test"])
        
        assert "name: custom" in cloud_init
        assert "docker" in cloud_init


def test_cli_with_write_files():
    """Test CLI with write_files configuration"""
    config = {
        "write_files": [
            {
                "path": "/etc/global.conf",
                "content": "global config"
            }
        ],
        "nodes": {
            "test": {
                "hostname": "test",
                "network": {},
                "write_files": [
                    {
                        "path": "/etc/node.conf",
                        "content": "node config"
                    }
                ]
            }
        }
    }
    
    with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml') as f:
        yaml.dump(config, f)
        f.flush()
        
        cli = CloudSeederCLI(Path(f.name))
        cli.load_config()
        cloud_init = cli.generate_cloud_init("test", config["nodes"]["test"])
        
        assert "/etc/global.conf" in cloud_init
        assert "/etc/node.conf" in cloud_init


def test_cli_with_selinux():
    """Test CLI with SELinux configuration"""
    config = {
        "selinux": {
            "mode": "permissive",
            "type": "mls"
        },
        "nodes": {
            "test": {
                "hostname": "test",
                "network": {}
            }
        }
    }
    
    with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml') as f:
        yaml.dump(config, f)
        f.flush()
        
        cli = CloudSeederCLI(Path(f.name))
        cli.load_config()
        cloud_init = cli.generate_cloud_init("test", config["nodes"]["test"])
        
        assert "SELINUX=permissive" in cloud_init
        assert "SELINUXTYPE=mls" in cloud_init


def test_cli_dhcp_network():
    """Test CLI with DHCP network (no static IP)"""
    config = {
        "nodes": {
            "dhcp-node": {
                "hostname": "dhcp-test",
                "network": {
                    "interface": "eth0"
                    # No ip_address - should use DHCP
                }
            }
        }
    }
    
    with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml') as f:
        yaml.dump(config, f)
        f.flush()
        
        cli = CloudSeederCLI(Path(f.name))
        cli.load_config()
        cloud_init = cli.generate_cloud_init("dhcp-node", config["nodes"]["dhcp-node"])
        
        # Should not have static network config
        assert "ethernets:" not in cloud_init or "dhcp4: true" in cloud_init


@patch('subprocess.run')
@patch('shutil.which')
def test_create_seed_iso_genisoimage(mock_which, mock_run):
    """Test ISO creation with genisoimage"""
    mock_which.return_value = "genisoimage"
    mock_run.return_value = MagicMock(returncode=0)
    
    with tempfile.TemporaryDirectory() as tmpdir:
        cli = CloudSeederCLI(Path("dummy.yaml"), output_dir=Path(tmpdir))
        cli._setup_directories()
        
        node_config = NodeConfig(
            name="test",
            hostname="test",
            fqdn="test.local",
            ip_address="192.168.1.10",
            gateway="192.168.1.1",
            netmask="255.255.255.0",
            dns_servers=["8.8.8.8"]
        )
        
        result = cli.create_seed_iso("test", "#cloud-config\n", node_config)
        
        assert result is True
        mock_run.assert_called_once()
        args = mock_run.call_args[0][0]
        assert args[0] == "genisoimage"
        assert "-volid" in args
        assert "cidata" in args


@patch('subprocess.run')
@patch('shutil.which')
def test_create_seed_iso_mkisofs(mock_which, mock_run):
    """Test ISO creation with mkisofs"""
    def which_side_effect(cmd):
        if cmd == "genisoimage":
            return None
        elif cmd == "mkisofs":
            return "mkisofs"
        return None
    
    mock_which.side_effect = which_side_effect
    mock_run.return_value = MagicMock(returncode=0)
    
    with tempfile.TemporaryDirectory() as tmpdir:
        cli = CloudSeederCLI(Path("dummy.yaml"), output_dir=Path(tmpdir))
        cli._setup_directories()
        
        result = cli.create_seed_iso("test", "#cloud-config\n")
        
        assert result is True
        args = mock_run.call_args[0][0]
        assert args[0] == "mkisofs"


@patch('shutil.which')
def test_create_seed_iso_no_tool(mock_which):
    """Test ISO creation with no tool available"""
    mock_which.return_value = None
    
    with tempfile.TemporaryDirectory() as tmpdir:
        cli = CloudSeederCLI(Path("dummy.yaml"), output_dir=Path(tmpdir))
        cli._setup_directories()
        
        result = cli.create_seed_iso("test", "#cloud-config\n")
        
        assert result is False


@patch('subprocess.run')
@patch('shutil.which')
def test_create_seed_iso_failure(mock_which, mock_run):
    """Test ISO creation failure"""
    mock_which.return_value = "genisoimage"
    mock_run.side_effect = subprocess.CalledProcessError(1, ["genisoimage"])
    
    with tempfile.TemporaryDirectory() as tmpdir:
        cli = CloudSeederCLI(Path("dummy.yaml"), output_dir=Path(tmpdir))
        cli._setup_directories()
        
        result = cli.create_seed_iso("test", "#cloud-config\n")
        
        assert result is False


def test_main_validate_only():
    """Test main function with validate-only"""
    config = {
        "nodes": {
            "test": {
                "hostname": "test",
                "network": {}
            }
        }
    }
    
    with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml', delete=False) as f:
        yaml.dump(config, f)
        config_path = f.name
    
    try:
        result = main([config_path, "--validate-only"])
        assert result == 0
    finally:
        Path(config_path).unlink()


def test_main_with_error():
    """Test main function with error"""
    result = main(["/nonexistent/config.yaml"])
    assert result == 1


def test_main_with_clean(tmp_path):
    """Test main function with clean option"""
    config = {
        "nodes": {
            "test": {
                "hostname": "test",
                "network": {}
            }
        }
    }
    
    config_file = tmp_path / "config.yaml"
    config_file.write_text(yaml.dump(config))
    output_dir = tmp_path / "output"
    
    # First generate files
    result = main([str(config_file), "-o", str(output_dir)])
    assert result == 0
    assert output_dir.exists()
    
    # Then clean and regenerate
    result = main([str(config_file), "-o", str(output_dir), "--clean"])
    assert result == 0


def test_node_config_defaults():
    """Test NodeConfig with default values"""
    node = NodeConfig(
        name="test",
        hostname="test",
        fqdn="test.local",
        ip_address="10.0.0.1",
        gateway="10.0.0.254",
        netmask="255.255.255.0",
        dns_servers=["8.8.8.8"]
    )
    
    assert node.interface == "ens192"  # Default value


def test_cli_datasource_config():
    """Test CLI with datasource configuration"""
    config = {
        "datasource": {
            "datasource_list": ["NoCloud", "None"],
            "NoCloud": {
                "seedfrom": "http://example.com/seed/"
            }
        },
        "nodes": {
            "test": {
                "hostname": "test",
                "network": {}
            }
        }
    }
    
    with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml') as f:
        yaml.dump(config, f)
        f.flush()
        
        cli = CloudSeederCLI(Path(f.name))
        cli.load_config()
        cloud_init = cli.generate_cloud_init("test", config["nodes"]["test"])
        
        assert "datasource_list:" in cloud_init
        assert "NoCloud" in cloud_init


def test_cli_commands():
    """Test CLI with bootcmd and runcmd"""
    config = {
        "bootcmd": ["echo 'global boot'"],
        "runcmd": ["echo 'global run'"],
        "nodes": {
            "test": {
                "hostname": "test",
                "network": {},
                "bootcmd": ["echo 'node boot'"],
                "runcmd": ["echo 'node run'"]
            }
        }
    }
    
    with tempfile.NamedTemporaryFile(mode='w', suffix='.yaml') as f:
        yaml.dump(config, f)
        f.flush()
        
        cli = CloudSeederCLI(Path(f.name))
        cli.load_config()
        cloud_init = cli.generate_cloud_init("test", config["nodes"]["test"])
        
        assert "echo 'global boot'" in cloud_init
        assert "echo 'global run'" in cloud_init
        assert "echo 'node boot'" in cloud_init
        assert "echo 'node run'" in cloud_init


def test_main_version():
    """Test main function with version flag"""
    with pytest.raises(SystemExit) as exc_info:
        main(["--version"])
    assert exc_info.value.code == 0